import UIKit
import Flutter

@UIApplicationMain
@objc class AppDelegate: FlutterAppDelegate, TapResearchSurveyDelegate, TapResearchRewardDelegate {
    
    var tapresearchPlacement = TRPlacement()
    var channel: FlutterMethodChannel!
    
    public func tapResearchSurveyWallOpened(with placement: TRPlacement) {
        channel.invokeMethod("tapResearchSurveyWallOpened", arguments: nil)
    }
    
    public func tapResearchSurveyWallDismissed(with placement: TRPlacement) {
        channel.invokeMethod("tapResearchSurveyWallDismissed", arguments: nil)
    }
    
    public func tapResearchDidReceive(_ reward: TRReward) {
        channel.invokeMethod("tapResearchDidReceive", arguments: reward.rewardAmount)
    }
    
    override func application(
        _ application: UIApplication,
        didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?
    ) -> Bool {
        let rootViewController : FlutterViewController = window?.rootViewController as! FlutterViewController
        channel = FlutterMethodChannel(name: "native", binaryMessenger: rootViewController as! FlutterBinaryMessenger)
        channel.setMethodCallHandler {(call: FlutterMethodCall, result: FlutterResult) -> Void in
            switch(call.method) {
            case "setUniqueUserIdentifier":
                if let args = call.arguments as? Dictionary<String, Any>,
                let userIdentifier = args["user_id"] as? String {
                    TapResearch.setUniqueUserIdentifier(userIdentifier)
                }
                break;
            case "initPlacement":
             if let args = call.arguments as? Dictionary<String, Any>,
                    let placementId = args["placement_id"] as? String {
            
                TapResearch.initPlacement(withIdentifier: placementId, placementBlock: { placement in
                    self.tapresearchPlacement = placement
                    if self.tapresearchPlacement.placementCode != PLACEMENT_CODE_SDK_NOT_READY {
                        if self.tapresearchPlacement.isSurveyWallAvailable == true {
                            self.channel.invokeMethod("isSurveyWallAvailable", arguments: true)
                        }
                    } else {
                        self.channel.invokeMethod("isSurveyWallAvailable", arguments: false)
                    }
                })
                } 
                break;
            case "showSurveyWall":
                self.tapresearchPlacement.showSurveyWall(with: nil)
                break;
            case "configure":     
                if let args = call.arguments as? Dictionary<String, Any>,
                    let apiToken = args["api_token"] as? String {
                    TapResearch.initWithApiToken(apiToken, delegate: nil)
                } 
                break;
            case "setNavigationBarText":
                if let args = call.arguments as? Dictionary<String, Any>,
                    let navBarText = args["navBarText"] as? String {
                    TapResearch.setNavigationBarText(navBarText)
                }      
                break;
            case "setNavigationBarColor":
                if let args = call.arguments as? Dictionary<String, Any>,
                    let red = args["red"] as? CGFloat, let green = args["green"] as? CGFloat, let blue = args["blue"] as? CGFloat, let alpha = args["alpha"] as? CGFloat {
                    TapResearch.setNavigationBarColor(UIColor.init(red: red, green: green, blue: blue, alpha: alpha))
                }
                break;
            case "setNavigationBarTextColor":
                if let args = call.arguments as? Dictionary<String, Any>,
                    let red = args["red"] as? CGFloat, let green = args["green"] as? CGFloat, let blue = args["blue"] as? CGFloat, let alpha = args["alpha"] as? CGFloat {
                    TapResearch.setNavigationBarTextColor(UIColor.init(red: red, green: green, blue: blue, alpha: alpha))
                }
                break;
            default: result(FlutterMethodNotImplemented)
            }
        }
        GeneratedPluginRegistrant.register(with: self)
        return super.application(application, didFinishLaunchingWithOptions: launchOptions)
        
    }

}
